import os
import shutil

from django.conf import settings
from django.test import TestCase

from localdevstorage.http import HttpStorage

import responses

DOMAIN = 'https://example.com'


class HttpStorageTest(TestCase):
    def tearDown(self):
        test_dir = os.path.join(settings.TEST_DIR, 'foo')
        if os.path.exists(test_dir):
            shutil.rmtree(test_dir)


    @responses.activate
    def test_http_fallback(self):
        responses.add(
            responses.GET,
            DOMAIN + '/foo/test_fallback',
            body='foo', status=200,
            content_type='text/plain'
        )
        storage = HttpStorage(fallback_domain=DOMAIN)
        f = storage.open('foo/test_fallback')
        self.assertIn('foo', str(f.read()))
        assert len(responses.calls) == 1

    @responses.activate
    def test_exists(self):
        responses.add(
            responses.HEAD,
            DOMAIN + '/foo/test_exists',
            body='foo', status=200,
            content_type='text/plain'
        )
        responses.add(
            responses.HEAD,
            DOMAIN + '/foo/test_does_not_exist',
            body='foo', status=404,
            content_type='text/plain'
        )
        storage = HttpStorage(fallback_domain=DOMAIN)
        self.assertTrue(storage.exists('foo/test_exists'))
        self.assertFalse(storage.exists('foo/test_does_not_exist'))

    @responses.activate
    def test_http_auth(self):
        responses.add(
            responses.GET,
            DOMAIN + '/foo/test_auth',
            body='foo', status=200,
            content_type='text/plain'
        )
        with self.settings(
                LOCALDEVSTORAGE_HTTP_PASSWORD='pw',
                LOCALDEVSTORAGE_HTTP_USERNAME='user'
        ):
            storage = HttpStorage(fallback_domain='https://example.com')
        f = storage.open('foo/test_auth')
        self.assertIn('Authorization', responses.calls[0].request.headers)



import os
from django.core.files.storage import FileSystemStorage

try:
    FileNotFoundError
except:
    FileNotFoundError = IOError


class BaseStorage(FileSystemStorage):
    def _open(self, name, mode='rb'):
        try:
            return super(BaseStorage, self)._open(name, mode)
        except FileNotFoundError:
            if 'w' in mode: # if writing, make sure the parent structure exists
                self._ensure_directory(name)

            try:
                try:
                    f = self._get(name)
                except IOError:
                    # if the underlying file doesn't exist, no matter.
                    pass
                else:
                    # if it does, write the contents locally
                    self._write(f, name)

            except Exception:
                pass
        return super(BaseStorage, self)._open(name, mode)

    def _exists_locally(self, name):
        return super(BaseStorage, self).exists(name)

    def exists(self, name):
        if self._exists_locally(name):
            return True
        return self._exists_upstream(name)

    def _ensure_directory(self, name):
        dirname = os.path.dirname(self.path(name))
        if not os.path.exists(dirname):
            os.makedirs(dirname)

    def _write(self, filelike, name):
        self._ensure_directory(name)
        f = open(self.path(name), mode='wb')
        f.write(filelike.read())

    def _fetch_local(self, name, force=False):
        if self._exists_locally(name) and not force:
            return

        return self._write(self._get(name), name)



import os

TEST_DIR = os.path.abspath(os.path.dirname(__file__))
MEDIA_ROOT = TEST_DIR

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

INSTALLED_APPS = [
    'django.contrib.staticfiles',
]

STATICFILES_FINDERS = [
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
]

STATIC_URL = '/static/'


STATIC_ROOT = os.path.join(TEST_DIR, 'static')

TEMPLATE_DIRS = (
    # Specifically choose a name that will not be considered
    # by app_directories loader, to make sure each test uses
    # a specific template without considering the others.
    os.path.join(TEST_DIR, 'test_templates'),
)


SECRET_KEY = "test"

PASSWORD_HASHERS = (
    'django.contrib.auth.hashers.UnsaltedMD5PasswordHasher',
)

MIDDLEWARE_CLASSES = []